@@ -1,3 +1,5 @@
+use std::cell::RefCell;
+
 use crate::{
     cost_model::transferred_byte_cycles,
     syscalls::{
@@ -21,6 +23,7 @@ pub struct LoadCellData<'a, DL> {
     resolved_cell_deps: &'a [CellMeta],
     group_inputs: &'a [usize],
     group_outputs: &'a [usize],
+    tracing_data_as_code_pages: &'a RefCell<Vec<(u64, u64)>>,
 }
 
 impl<'a, DL: CellDataProvider + 'a> LoadCellData<'a, DL> {
@@ -31,6 +34,7 @@ impl<'a, DL: CellDataProvider + 'a> LoadCellData<'a, DL> {
         resolved_cell_deps: &'a [CellMeta],
         group_inputs: &'a [usize],
         group_outputs: &'a [usize],
+        tracing_data_as_code_pages: &'a RefCell<Vec<(u64, u64)>>,
     ) -> LoadCellData<'a, DL> {
         LoadCellData {
             data_loader,
@@ -39,6 +43,7 @@ impl<'a, DL: CellDataProvider + 'a> LoadCellData<'a, DL> {
             resolved_cell_deps,
             group_inputs,
             group_outputs,
+            tracing_data_as_code_pages,
         }
     }
 
@@ -111,8 +116,12 @@ impl<'a, DL: CellDataProvider + 'a> LoadCellData<'a, DL> {
             Some(data),
             0,
         )?;
+        // tracing data as code page index
+        self.tracing_data_as_code_pages
+            .borrow_mut()
+            .push((addr, memory_size));
 
-        machine.add_cycles(transferred_byte_cycles(memory_size))?;
+        machine.add_cycles_no_checking(transferred_byte_cycles(memory_size))?;
         machine.set_register(A0, Mac::REG::from_u8(SUCCESS));
         Ok(())
     }
@@ -133,7 +142,7 @@ impl<'a, DL: CellDataProvider + 'a> LoadCellData<'a, DL> {
             .ok_or(VMError::Unexpected)?;
 
         let wrote_size = store_data(machine, &data)?;
-        machine.add_cycles(transferred_byte_cycles(wrote_size))?;
+        machine.add_cycles_no_checking(transferred_byte_cycles(wrote_size))?;
         machine.set_register(A0, Mac::REG::from_u8(SUCCESS));
         Ok(())
     }
