@@ -2,12 +2,13 @@ use crate::ScriptError;
 use ckb_error::Error;
 use ckb_types::{
     core::{Cycle, ScriptHashType},
-    packed::{Byte32, Script},
+    packed::Script,
 };
 use ckb_vm::{
     machine::{VERSION0, VERSION1},
+    memory::{FLAG_EXECUTABLE, FLAG_FREEZED},
     snapshot::{make_snapshot, Snapshot},
-    SupportMachine, ISA_B, ISA_IMC, ISA_MOP,
+    CoreMachine as _, Memory, SupportMachine, ISA_B, ISA_IMC, ISA_MOP, RISCV_PAGESIZE,
 };
 use serde::{Deserialize, Serialize};
 use std::convert::TryFrom;
@@ -205,12 +206,10 @@ impl fmt::Display for ScriptGroupType {
 /// Struct specifies which script has verified so far.
 /// Snapshot is lifetime free, but capture snapshot need heavy memory copy
 pub struct TransactionSnapshot {
-    /// current suspended script
-    pub current: (ScriptGroupType, Byte32),
-    /// remain script groups to verify
-    pub remain: Vec<(ScriptGroupType, Byte32)>,
+    /// current suspended script index
+    pub current: usize,
     /// vm snapshot
-    pub snap: Option<Snapshot>,
+    pub snap: Option<(Snapshot, Cycle)>,
     /// current consumed cycle
     pub current_cycles: Cycle,
     /// limit cycles when snapshot create
@@ -220,16 +219,18 @@ pub struct TransactionSnapshot {
 /// Struct specifies which script has verified so far.
 /// State lifetime bound with vm machine.
 pub struct TransactionState<'a> {
-    /// current suspended script
-    pub current: (ScriptGroupType, Byte32),
-    /// remain script groups to verify
-    pub remain: Vec<(ScriptGroupType, Byte32)>,
+    /// current suspended script index
+    pub current: usize,
     /// vm state
-    pub vm: ResumableMachine<'a>,
+    pub vm: Option<ResumableMachine<'a>>,
     /// current consumed cycle
     pub current_cycles: Cycle,
     /// limit cycles
     pub limit_cycles: Cycle,
+    /// enable snapshot page dirty flags
+    pub enable_backup_page_flags: bool,
+    /// tracing data as code page index
+    pub flags_tracing: Vec<(u64, u64)>,
 }
 
 impl TransactionState<'_> {
@@ -266,30 +267,53 @@ impl TryFrom<TransactionState<'_>> for TransactionSnapshot {
     fn try_from(state: TransactionState<'_>) -> Result<Self, Self::Error> {
         let TransactionState {
             current,
-            remain,
-            mut vm,
+            vm,
             current_cycles,
             limit_cycles,
+            enable_backup_page_flags,
+            flags_tracing,
         } = state;
 
-        // we should not capture snapshot if load program failed by exceeded cycles
-        let (snap, current_cycles) = if vm.program_loaded {
-            let vm_cycles = vm.cycles();
-            (
-                Some(make_snapshot(&mut vm.machine.machine).map_err(|e| {
-                    ScriptError::VMInternalError(format!("{:?}", e)).unknown_source()
-                })?),
-                current_cycles.checked_add(vm_cycles).ok_or_else(|| {
-                    ScriptError::CyclesOverflow(current_cycles, vm_cycles).unknown_source()
-                })?,
-            )
+        let (snap, current_cycles) = if let Some(mut vm) = vm {
+            // we should not capture snapshot if load program failed by exceeded cycles
+            if vm.program_loaded {
+                let vm_cycles = vm.cycles();
+                // To be consistent with the mainnet, add this flag to enable this behavior after hardfork
+                if !enable_backup_page_flags {
+                    for (addr, memory_size) in flags_tracing {
+                        let mut current_addr = addr;
+                        while current_addr < addr + memory_size {
+                            let page = current_addr / RISCV_PAGESIZE as u64;
+                            vm.machine
+                                .machine
+                                .memory_mut()
+                                .clear_flag(page, FLAG_EXECUTABLE | FLAG_FREEZED)
+                                .map_err(|e| {
+                                    ScriptError::VMInternalError(format!("{:?}", e))
+                                        .unknown_source()
+                                })?;
+                            current_addr += RISCV_PAGESIZE as u64;
+                        }
+                    }
+                }
+                (
+                    Some((
+                        make_snapshot(&mut vm.machine.machine).map_err(|e| {
+                            ScriptError::VMInternalError(format!("{:?}", e)).unknown_source()
+                        })?,
+                        vm_cycles,
+                    )),
+                    current_cycles,
+                )
+            } else {
+                (None, current_cycles)
+            }
         } else {
             (None, current_cycles)
         };
 
         Ok(TransactionSnapshot {
             current,
-            remain,
             snap,
             current_cycles,
             limit_cycles,
@@ -298,6 +322,7 @@ impl TryFrom<TransactionState<'_>> for TransactionSnapshot {
 }
 
 /// Enum represent resumable verify result
+#[allow(clippy::large_enum_variant)]
 #[derive(Debug)]
 pub enum VerifyResult<'a> {
     /// Completed total cycles
@@ -310,7 +335,6 @@ impl std::fmt::Debug for TransactionSnapshot {
     fn fmt(&self, f: &mut ::core::fmt::Formatter) -> std::fmt::Result {
         f.debug_struct("TransactionSnapshot")
             .field("current", &self.current)
-            .field("remain", &self.remain)
             .field("current_cycles", &self.current_cycles)
             .field("limit_cycles", &self.limit_cycles)
             .finish()
@@ -321,7 +345,6 @@ impl std::fmt::Debug for TransactionState<'_> {
     fn fmt(&self, f: &mut ::core::fmt::Formatter) -> std::fmt::Result {
         f.debug_struct("TransactionState")
             .field("current", &self.current)
-            .field("remain", &self.remain)
             .field("current_cycles", &self.current_cycles)
             .field("limit_cycles", &self.limit_cycles)
             .finish()
